#!/usr/bin/python

import sys

NetworkKey = [ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 ]

NetworkKey = [ 0xB9, 0xA5, 0x21, 0xFB, 0xBD, 0x72, 0xC3, 0x45 ]

if sum(NetworkKey)==0:
    print "Please fill in the ANT+ Network Key."
    sys.exit(-1)

import ant
import autoant_writer
import time
import math

antclass=autoant_writer.AutoAntWriter

class CinQo(antclass):
    def __init__(self, filename):
        antclass.__init__(self, quiet=1)
        try:
            self.open_ants(filename)
            print "Saving autoant script as \"%s\""%filename
        except AttributeError:
            pass

        self.auto_init()

        self.throbber="-\|/"
        self.dropping=False

        try:
            self.close_channel(0)
        except ant.AntWrongResponseException:
            pass
            
        self.assign_channel(channel=0,
                            type=0x10,
                            network=1)
        self.set_network_key(network=1, key=NetworkKey)
        self.set_channel_id(channel=0,
                            device=12345,
                            device_type_id=11,
                            man_id=5)
        self.set_channel_freq(0, 57)


        self.set_channel_period(0, 8182)
        
        self.set_channel_search_timeout(0, 40)
        self.open_channel(0)

        self.event_counter=0
        self.event_counter2=0

        self.torque_accum=0
        self.t0=time.time();
        self.last_time_ticks=-1
        
        self.power_accum=0

        self.standard_event_counter=0

    def get_2048_clk(self):
        return int(2048.0*(time.time()-self.t0))

    def crank_event(self,power=0):
               
        time_ticks=self.get_2048_clk()

        cadence=2048*60.0/(time_ticks-self.last_time_ticks)

        #rint "cadence",cadence
        if cadence > 255: cadence=255

        angular_velo = 2*math.pi / ((time_ticks-self.last_time_ticks)/2048.0)

        torque = power / angular_velo

        self.torque_accum+=32*torque
 
        #rint self.torque_accum, self.event_counter

        self.power_accum+=power
        
        self.standard_event_counter+=1
        
        #if self.event_counter%16 > 8:
        #    balance = 51
        #else:
        #    balance = 0xff

        balance = 50

        #rint "balance", balance

        #balance=self.event_counter%100

        self.standard_data=[0x10,
                            (self.event_counter+128)&0xff,
                            0x80|balance,
                            int(cadence), #0xff, # instant cadence
                            int(self.power_accum)&0xff,
                            (int(self.power_accum)>>8)&0xff,
                            int(power)&0xff,
                            (int(power)>>8)&0xff]
        self.crank_torque_data=[0x12,
                                self.event_counter,
                                self.event_counter, # crank ticks
                                0xff, # instant cadence
                                time_ticks&0xff,
                                (time_ticks>>8)&0xff,
                                int(self.torque_accum)&0xff,
                                (int(self.torque_accum)>>8)&0xff]

        self.event_counter=(self.event_counter+1)%0xff
        self.last_time_ticks=time_ticks


    def cinqo_broadcast(self, standard=False):
        if self.dropping:
            sys.stdout.write("d")
        elif standard:
            sys.stdout.write("+")
        else:
            sys.stdout.write(".")
        #print "%5.2f"%time.time()
 
        if standard:
            #print "sending standard data"
            self.send_broadcast_data(0,self.standard_data)
        else:
            #print "sending crank torque data"
            self.send_broadcast_data(0,self.crank_torque_data)

    def wait_tx(self):
        sys.stdout.write(self.throbber[0]+"\b")
        sys.stdout.flush()
        self.throbber=self.throbber[1:]+self.throbber[0]
        
        self.sp.setTimeout(0.05)

        while 1:
            try:
                resp=self.receive_message() 
                if not resp:
                    continue
                #print resp.name

                if resp.name=='calibration_request':
                    self.calibrate()
                if resp.name=='event_tx':
                    return

            except ant.AntNoDataException:
                pass

            #wait_for_response([['event_tx', {'channel':0}],
            #                             ['calibration_request', {'channel':0}]],15)
            
        
    def send_one_second(self,watts):
        message_counter=0
        self.crank_event(watts)        
        for i in range(4):
            self.wait_tx()            
            self.cinqo_broadcast(message_counter==1)            
            message_counter+=1
               
    def calibrate(self):
        print "no cal defined"

    def five_minutes_120RPM_flatwave(self):
        t0=time.time()
        while time.time() < (t0 + 60*5):           
            self.crank_event(200)        
            self.cinqo_broadcast(standard=False)            
            for i in range(2): self.wait_tx()
            self.crank_event(200)        
            self.cinqo_broadcast(standard=False)            
            for i in range(2): self.wait_tx()

    def five_minutes_40RPM_flatwave(self):
        t0=time.time()
        while time.time() < (t0 + 60*5):           
            self.crank_event(200)        
            self.cinqo_broadcast(standard=False)            
            for i in range(6): self.wait_tx()
            self.crank_event(200)        
            self.cinqo_broadcast(standard=False)            
            for i in range(6): self.wait_tx()

    def five_minutes_120RPM_rf_dropout(self):
        # At 120 RPM, cadence should rollover at 2:33 (Ant+ Bike Power Table 9-4)
        # At 200 W. torque should rollover at 32 seconds. (Ant+ Bike Power Table 9-6) 

        dropout_time=10 # seconds
        tx_time=5 # seconds

        t0=time.time()        
        
        def check_dropout():
            t=time.time()-t0
            t=t%(tx_time+dropout_time)
            if (t <= dropout_time):
                #print "drop"
                self.set_channel_freq(0, 50)
                self.dropping=True
            else:
                #print "no drop"
                self.set_channel_freq(0, 57)
                self.dropping=False
               
        while time.time() < (t0 + 60*5):           
            self.crank_event(200)        
            self.cinqo_broadcast(standard=False)
            for i in range(2): 
                self.wait_tx()
                check_dropout()

    def five_minutes_120RPM_rf_dropout2(self):
        # At 120 RPM, cadence should rollover at 2:33 (Ant+ Bike Power Table 9-4)
        # At 200 W. torque should rollover at 32 seconds. (Ant+ Bike Power Table 9-6) 

        dropout_time=12 # messages
        tx_time=12 # messages

        t0=time.time()        
        
        self.x=0

        def check_dropout():
            
            if (self.x%(dropout_time+tx_time))<tx_time:
                #print "no drop"
                self.set_channel_freq(0, 57)
                self.dropping=False
            else:
                #print "drop"
                self.set_channel_freq(0, 50)
                self.dropping=True
            self.x+=1
               
        while time.time() < (t0 + 60*5):           
            self.crank_event(200)        
            self.cinqo_broadcast(standard=False)
            for i in range(2): 
                self.wait_tx()
                check_dropout()

    def five_minutes_120RPM_squarewave(self):
        t0=time.time()
        while time.time() < (t0 + 60*5):           
            self.crank_event(100)        
            self.cinqo_broadcast(standard=False)            
            for i in range(2): self.wait_tx()
            self.crank_event(300)        
            self.cinqo_broadcast(standard=False)            
            for i in range(2): self.wait_tx()

    def five_minutes_120RPM_squarewave_stdpower(self):
        t0=time.time()
        while time.time() < (t0 + 60*5):           
            self.crank_event(100)        
            self.cinqo_broadcast(standard=True)            
            for i in range(2): self.wait_tx()
            self.crank_event(300)        
            self.cinqo_broadcast(standard=True)            
            for i in range(2): self.wait_tx()

    def five_minutes_60RPM_20RPM_squarewave(self):
        t0=time.time()
        while time.time() < (t0 + 60*5):           
            self.crank_event(200.0/3)        
            self.cinqo_broadcast(standard=False)            
            for i in range(4): self.wait_tx()
            self.crank_event(600)        
            self.cinqo_broadcast(standard=False)            
            for i in range(4*3): self.wait_tx()


    def five_minutes_60RPM_6RPM(self):
        # twenty seconds of 60 RPM, 400W (twenty revs)
        # twenty seconds of 6 RPM, 0W (two revs)
        t0=time.time()
        while time.time() < (t0 + 60*5):           
            for i in range(20):
                self.wait_tx() 
                self.wait_tx()
                self.crank_event(400)      
                self.cinqo_broadcast(standard=False)            
                self.wait_tx() 
                self.cinqo_broadcast(standard=True)            
                self.wait_tx() 
                self.cinqo_broadcast(standard=False)            

            for j in range(2): 
                for i in range(10*4):
                    self.wait_tx()

                self.crank_event(0)                   
                self.cinqo_broadcast(standard=False)            

if __name__=="__main__":
    try:
        val=sys.argv[1]
        if not val in "12345678a": 
            raise IndexError

    except IndexError:
        print "\n%s -- synthesize a power meter signal with a USB stick"%sys.argv[0]
        print "\nUse like this:"
        print "  %s OPTION"%sys.argv[0]
        print "\nWhere OPTION is one of:"
        print "\t1 - 120 RPM / 200 W constant power"
        print "\t2 - 120 RPM / 100 W alternating with 120 RPM / 300 W"
        print "\t3 -  60 RPM / 600 W alternating with  20 RPM / 66.7W"
        print "\t4 - 120 RPM / 100 W alternating with 120 RPM / 300 W (with standard power messages)"
        print "\t5 - 120 RPM / 200 W constant power with ten second RF dropouts"
        print "\t6 - 120 RPM / 200 W constant power with three second RF dropouts"
        print "\t7 - 60 RPM / 400 W constant power alternating 6 RPM / 0 W, ten seconds per."
        print "\t8 - 40 RPM / 200 W constant power"
        
        print "  (all modes average 200W)"
        sys.exit(-1)

    def go(val):
        try:
            c=CinQo("ct_sim_%s.ants"%val)

            {"1":c.five_minutes_120RPM_flatwave,
             "2":c.five_minutes_120RPM_squarewave,
             "3":c.five_minutes_60RPM_20RPM_squarewave,
             "4":c.five_minutes_120RPM_squarewave_stdpower,
             "5":c.five_minutes_120RPM_rf_dropout,
             "6":c.five_minutes_120RPM_rf_dropout2,
             "7":c.five_minutes_60RPM_6RPM,
             "8":c.five_minutes_40RPM_flatwave}[val]()

            c.close_channel(0)

        except KeyboardInterrupt:
            try:
                c.close_channel(0)
            except ant.AntWrongResponseException:
                pass
            raise
            

    if val=="a":
        for v in "1234567":
            go(v)
    else:
        go(val)




